require "podsorz/core/PodsOrz/pods_repo"
require "podsorz/core/Binary/binary_manager"
require "podsorz/core/PodsOrz/pods_git_manager"


module PodsOrz
	def self.binary_command
		desc "Binary command, static library related command"
		command :binary do |binary|
			binary.desc "package pods(which in PodsOrzConfig) into static libraries"
			binary.command :package do |package|
				package.action do |global_options,options,args|
					dir_path = global_options[:path]
					prepare_work(dir_path)

					git_manager = PodsOrz::PodsGitManager.new(dir_path)
					result_hash = filter_binary_pods_list(dir_path,git_manager, :package)
					package_list = result_hash[:package]

					binary_manager = PodsOrz::BinaryManager.new(dir_path)
					binary_manager.package(package_list)
				end
					
			end



			binary.desc "show whether remote exist static libraries on current version"
			binary.command :show do |show|
				show.action do |global_options,options,args|
					dir_path = global_options[:path]
					prepare_work(dir_path)

					git_manager = PodsOrz::PodsGitManager.new(dir_path)
					filter_binary_pods_list(dir_path,git_manager, :show)
				end
					
			end


			binary.desc "publish binary repo"
			binary.command :publish do |publish|
				publish.action do |global_options,options,args|
					dir_path = global_options[:path]
					prepare_work(dir_path)

					git_manager = PodsOrz::PodsGitManager.new(dir_path)
					result_hash = filter_binary_pods_list(dir_path,git_manager, :package)
					package_list = result_hash[:package]

					binary_manager = PodsOrz::BinaryManager.new(dir_path)
					binary_manager.publish(package_list)
				end
			end


			binary.default_command :show
		end
	end

	def self.prepare_work(dir_path)
		pods_repo = PodsOrz::PodsRepo.new(dir_path)
		pods_repo.check_repo_exist()
	end


	def self.filter_binary_pods_list(dir_path,git_manager, cmd_type)
		result_hash = {
			:binary => [],
			:package => []
		}

		#1.获取静态库加载配置表
		is_all_binary = git_manager.orzconfig_parse.is_all_binary

		origin_binary_list = []

		unless cmd_type == :show
			if is_all_binary
				totoal_model_list = git_manager.podfile_io.filter_binary_pod_models()
				totoal_model_list.each do |model|
					origin_binary_list << model.name.to_s
				end
			else
				origin_binary_list = git_manager.orzconfig_parse.static_lib_list if cmd_type == :binary
				origin_binary_list = git_manager.orzconfig_parse.will_package_list if cmd_type == :package
			end

		else 
			totoal_model_list = git_manager.podfile_io.total_pod_models
			totoal_model_list.each do |model|
				origin_binary_list << model.name.to_s
			end
		end
		

		return result_hash if origin_binary_list.empty?


		empty_binary_list = []
		exist_binary_list = []

		has_binary_list = []
		can_package_list = []

		#2.查询Podfile_orz是否为已发版本状态
		binary_repo = PodsOrz::BinaryRepo.new(dir_path)
		binary_repo.check_binary_repo_exist()

		origin_binary_list.each do |pod|
			pod_version = git_manager.podfile_io.get_podfile_pod_version(pod)

			if pod_version.empty?
				branch = git_manager.podfile_io.get_podfile_pod_branch(pod)
				unless branch.empty?
					empty_binary_list << "#{pod} no source-code version, :branch => #{branch}"
				else
					empty_binary_list << "#{pod} no source-code version, :branch => 'master'" 
				end

				next
			end

			has_remote = binary_repo.check_binary_pod_exist(pod, pod_version)
			unless has_remote
				empty_binary_list << "#{pod} no binary version:(#{pod_version})"
			else
				exist_binary_list << "#{pod}:(#{pod_version})"
				has_binary_list << pod
			end

			if !pod_version.empty? && !has_remote
				can_package_list << pod
			end
		end

		Logger.highlight("[cmd:#{cmd_type}] Exist static library pods :")
		exist_binary_list.each do |item|
			puts item.colorize(:green)
		end

		Logger.separator

		Logger.warning("[cmd:#{cmd_type}] Not exist static library pods:")
		empty_binary_list.each do |item|
			puts item.colorize(:yellow)
		end

		result_hash[:binary] = has_binary_list
		result_hash[:package] = can_package_list
		

		result_hash

	end

end