require "podsorz/core/PodsOrz/pods_detector"
require "podsorz/core/PodsOrz/pods_git_manager"

module PodsOrz
	def self.sync_command
		desc "Sync current_branch latest code from origin/current_branch or --rebase=‘other_branch’ "
		command :sync do |sync|
			sync.flag [:r, :rebase], :desc => "set rebase command on 'other_branch' "

			sync.example "podsorz sync", desc:"current_branch 'git pull --rebase' "
			sync.example "podsorz sync -r release", desc:"1.current_branch 'git pull --rebase', 2.sync commit from release"
			sync.example "podsorz sync --rebase=feature/xq_2.0.0", desc:"1.current_branch 'git pull --rebase', 2.sync commit from feature/xq_2.0.0"

			sync.action do |global_options, options, args|
				other_branch = options[:rebase]
				other_branch = other_branch.to_s.strip.chomp

				dir_path = global_options[:path]

				detector = PodsOrz::PodsDetector.new(dir_path)
				detector.start_detector()

				git_manager = PodsOrz::PodsGitManager.new(dir_path)
				is_same = git_manager.ensure_pods_branch()
				exit() unless is_same

				git_manager.cmd_sync_pod(other_branch)
			end
		end
	end

end